@@ -13,7 +13,7 @@
 #' agg <- range_stat_aggregator$new(
 #'   ylwr = min, y = mean, yupr = max, interleave_gaps = TRUE
 #' )
-#' d_agg <- agg$aggregate(noise_fluct$time, noise_fluct$f500, 100)
+#' d_agg <- agg$aggregate(nanotime::as.nanotime(noise_fluct$time), noise_fluct$f500, 100)
 #' plotly::plot_ly(x = d_agg$x, y = d_agg$y, type = "scatter", mode = "lines") %>%
 #'   plotly::add_trace(x = d_agg$x, y = d_agg$ylwr, type = "scatter", mode = "lines")%>%
 #'   plotly::add_trace(x = d_agg$x, y = d_agg$yupr, type = "scatter", mode = "lines")
@@ -24,7 +24,7 @@ range_stat_aggregator <- R6::R6Class(
   public = list(
     #' @description
     #' Constructor of the aggregator.
-    #' @param interleave_gaps,coef_gap,NA_position,accepted_datatype,...
+    #' @param interleave_gaps,coef_gap,NA_position,...
     #' Arguments pass to the constructor of \code{aggregator} object.
     #' @param yupr,y,ylwr Functions.
     #' Statistical values are calculated using this function.
@@ -33,7 +33,7 @@ range_stat_aggregator <- R6::R6Class(
     initialize = function(
       ...,
       ylwr = min, y = mean, yupr = max,
-      interleave_gaps, coef_gap, NA_position, accepted_datatype
+      interleave_gaps, coef_gap, NA_position
     ) {
       args <- c(as.list(environment()), list(...))
       do.call(super$initialize, args)
@@ -56,6 +56,7 @@ range_stat_aggregator <- R6::R6Class(
     }
   ),
   private = list(
+    accepted_datatype = c("numeric", "integer", "character", "factor", "logical"),
     ylwr = NULL,
     y    = NULL,
     yupr = NULL,
